@@ -2,12 +2,17 @@ package Test2::Event::Exception;
 use strict;
 use warnings;
 
-our $VERSION = '1.302086';
+our $VERSION = '1.302186';
 
 
 BEGIN { require Test2::Event; our @ISA = qw(Test2::Event) }
 use Test2::Util::HashBase qw{error};
 
+sub init {
+    my $self = shift;
+    $self->{+ERROR} = "$self->{+ERROR}";
+}
+
 sub causes_fail { 1 }
 
 sub summary {
@@ -18,6 +23,22 @@ sub summary {
 
 sub diagnostics { 1 }
 
+sub facet_data {
+    my $self = shift;
+    my $out = $self->common_facet_data;
+
+    $out->{errors} = [
+        {
+            tag     => 'ERROR',
+            fail    => 1,
+            details => $self->{+ERROR},
+        }
+    ];
+
+    return $out;
+}
+
+
 1;
 
 __END__
@@ -55,6 +76,10 @@ The reason for the exception.
 
 =back
 
+=head1 CAVEATS
+
+Be aware that all exceptions are stringified during construction.
+
 =head1 SOURCE
 
 The source code repository for Test2 can be found at
@@ -78,7 +103,7 @@ F<http://github.com/Test-More/test-more/>.
 
 =head1 COPYRIGHT
 
-Copyright 2017 Chad Granum E<lt>exodist@cpan.orgE<gt>.
+Copyright 2020 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
