@@ -2,7 +2,7 @@ package Test2::Formatter;
 use strict;
 use warnings;
 
-our $VERSION = '1.302085';
+our $VERSION = '1.302186';
 
 
 my %ADDED;
@@ -14,6 +14,13 @@ sub import {
     Test2::API::test2_formatter_add($class);
 }
 
+sub new_root {
+    my $class = shift;
+    return $class->new(@_);
+}
+
+sub supports_tables { 0 }
+
 sub hide_buffered { 1 }
 
 sub terminate { }
@@ -56,12 +63,20 @@ A formatter is any package or object with a C<write($event, $num)> method.
 
     sub finalize { }
 
+    sub supports_tables { return $BOOL }
+
+    sub new_root {
+        my $class = shift;
+        ...
+        $class->new(@_);
+    }
+
     1;
 
 The C<write> method is a method, so it either gets a class or instance. The two
 arguments are the C<$event> object it should record, and the C<$assert_num>
 which is the number of the current assertion (ok), or the last assertion if
-this even is not itself an assertion. The assertion number may be any integer 0
+this event is not itself an assertion. The assertion number may be any integer 0
 or greater, and may be undefined in some cases.
 
 The C<hide_buffered()> method must return a boolean. This is used to tell
@@ -81,6 +96,15 @@ The C<finalize> method is always the last thing called on the formatter, I<<
 except when C<terminate> is called for a Bail event >>. It is passed the
 following arguments:
 
+The C<supports_tables> method should be true if the formatter supports directly
+rendering table data from the C<info> facets. This is a newer feature and many
+older formatters may not support it. When not supported the formatter falls
+back to rendering C<detail> instead of the C<table> data.
+
+The C<new_root> method is used when constructing a root formatter. The default
+is to just delegate to the regular C<new()> method, most formatters can ignore
+this.
+
 =over 4
 
 =item * The number of tests that were planned
@@ -95,6 +119,12 @@ following arguments:
 
 =back
 
+The C<new_root> method is called when C<Test2::API::Stack> Initializes the root
+hub for the first time. Most formatters will simply have this call C<<
+$class->new >>, which is the default behavior. Some formatters however may want
+to take extra action during construction of the root formatter, this is where
+they can do that.
+
 =head1 SOURCE
 
 The source code repository for Test2 can be found at
@@ -118,7 +148,7 @@ F<http://github.com/Test-More/test-more/>.
 
 =head1 COPYRIGHT
 
-Copyright 2017 Chad Granum E<lt>exodist@cpan.orgE<gt>.
+Copyright 2020 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
