@@ -2,14 +2,16 @@ package Test2::IPC;
 use strict;
 use warnings;
 
-our $VERSION = '1.302086';
+our $VERSION = '1.302186';
 
 
 use Test2::API::Instance;
 use Test2::Util qw/get_tid/;
 use Test2::API qw{
+    test2_in_preload
     test2_init_done
     test2_ipc
+    test2_has_ipc
     test2_ipc_enable_polling
     test2_pid
     test2_stack
@@ -17,14 +19,27 @@ use Test2::API qw{
     context
 };
 
+# Make sure stuff is finalized before anyone tried to fork or start a new thread.
+{
+    # Avoid warnings if things are loaded at run-time
+    no warnings 'void';
+    INIT {
+        use warnings 'void';
+        context()->release() unless test2_in_preload();
+    }
+}
+
 use Carp qw/confess/;
 
 our @EXPORT_OK = qw/cull/;
 BEGIN { require Exporter; our @ISA = qw(Exporter) }
 
-sub import {
-    goto &Exporter::import unless test2_init_done();
+sub unimport { Test2::API::test2_ipc_disable() }
 
+sub import {
+    goto &Exporter::import if test2_has_ipc || !test2_init_done();
+
+    confess "IPC is disabled" if Test2::API::test2_ipc_disabled();
     confess "Cannot add IPC in a child process (" . test2_pid() . " vs $$)" if test2_pid() != $$;
     confess "Cannot add IPC in a child thread (" . test2_tid() . " vs " . get_tid() . ")"  if test2_tid() != get_tid();
 
@@ -95,6 +110,11 @@ You should C<use Test2::IPC;> as early as possible in your test file. If you
 import this module after API initialization it will attempt to retrofit IPC
 onto the existing hubs.
 
+=head2 DISABLING IT
+
+You can use C<no Test2::IPC;> to disable IPC for good. You can also use the
+T2_NO_IPC env var.
+
 =head1 EXPORTS
 
 All exports are optional.
@@ -130,7 +150,7 @@ F<http://github.com/Test-More/test-more/>.
 
 =head1 COPYRIGHT
 
-Copyright 2017 Chad Granum E<lt>exodist@cpan.orgE<gt>.
+Copyright 2020 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
